import CloseIcon from "@mui/icons-material/Close";
import CloudDownloadIcon from "@mui/icons-material/CloudDownload";
import DownloadIcon from "@mui/icons-material/Download";
import {
  AppBar,
  Button,
  Dialog,
  IconButton,
  Paper,
  Toolbar,
  Typography,
} from "@mui/material";
import { filesize } from "filesize";
import React from "react";
import { FileApi, UploadedFile } from "../api/FileApi";

export function FileViewerDialog(p: {
  open: boolean;
  onClose: () => void;
  file: UploadedFile;
}): React.ReactElement {
  if (!p.open) return <></>;
  return (
    <Dialog open={p.open} onClose={p.onClose} fullScreen>
      <AppBar sx={{ position: "relative" }}>
        <Toolbar>
          <IconButton
            edge="start"
            color="inherit"
            onClick={p.onClose}
            aria-label="close"
            autoFocus
          >
            <CloseIcon />
          </IconButton>
          <Typography sx={{ ml: 2, flex: 1 }} variant="h6" component="div">
            {p.file.file_name} ({filesize(p.file.file_size)})
          </Typography>
          <a
            href={FileApi.DownloadURL(p.file, true)}
            target="_blank"
            referrerPolicy="no-referrer"
          >
            <Button
              color="info"
              onClick={p.onClose}
              startIcon={<DownloadIcon />}
            >
              Download
            </Button>
          </a>
        </Toolbar>
      </AppBar>
      <FileViewer
        url={FileApi.DownloadURL(p.file)}
        downloadUrl={FileApi.DownloadURL(p.file, true)}
        {...p.file}
      />
    </Dialog>
  );
}

type ViewerProps = {
  url: string;
  downloadUrl: string;
} & UploadedFile;

function FileViewer(p: ViewerProps): React.ReactElement {
  // Image
  if (p.mime_type.startsWith("image/")) return <ImageViewer {...p} />;
  // PDF
  else if (p.mime_type === "application/pdf") return <PDFViewer {...p} />;
  // Default viewer
  else return <DefaultViewer {...p} />;
}

function ImageViewer(p: ViewerProps): React.ReactElement {
  return (
    <img
      src={p.url}
      style={{ maxWidth: "100%", width: "fit-content", margin: "auto" }}
    />
  );
}

function PDFViewer(p: ViewerProps): React.ReactElement {
  // eslint-disable-next-line react-dom/no-missing-iframe-sandbox
  return <iframe style={{ flex: 1 }} src={p.url} />;
}

function DefaultViewer(p: ViewerProps): React.ReactElement {
  return (
    <Paper
      elevation={3}
      style={{
        margin: "10px",
        padding: "10px",
        display: "flex",
        flexDirection: "column",
        justifyContent: "center",
        alignItems: "center",
      }}
    >
      <Typography variant="h5" gutterBottom>
        <CloudDownloadIcon fontSize="large" />
      </Typography>
      <Typography variant="caption" gutterBottom>
        {filesize(p.file_size)}
      </Typography>
      <a href={p.downloadUrl} target="_blank" referrerPolicy="no-referrer">
        <Button variant="outlined">Download</Button>
      </a>
    </Paper>
  );
}
